﻿#region Licence
// Copyright (c) 2011 BAX Services Bartosz Adamczewski
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CSpec.Tests.Examples.Calculator
{
    /// <summary>
    /// A very simple calc interface
    /// with value carry over.
    /// </summary>
    public interface ICalc
    {
        /// <summary>
        /// Adds a value to the total.
        /// </summary>
        /// <param name="a"></param>
        /// <returns></returns>
        int Sum(int a);
        /// <summary>
        /// Substracts a value from total.
        /// </summary>
        /// <param name="a"></param>
        /// <returns></returns>
        int Sub(int a);
        /// <summary>
        /// Takes a total to a power of a given value.
        /// </summary>
        /// <param name="a"></param>
        /// <returns></returns>
        int Pow(int a);
        /// <summary>
        /// Divides the total by a given value.
        /// </summary>
        /// <param name="a"></param>
        /// <returns></returns>
        int Div(int a);

        /// <summary>
        /// Gets/Sets the total value.
        /// </summary>
        int Total { get; set; }

        /// <summary>
        /// Rests the total.
        /// </summary>
        void Reset();
        
    }
}
