﻿#region Licence
// Copyright (c) 2011 BAX Services Bartosz Adamczewski
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;
using System.Collections;

namespace CSpec.Proxy
{
    /// <summary>
    /// Represents a descriptor of a proxied method.
    /// </summary>
    public class ProxyMethod
    {
        /// <summary>
        /// Gets/Sets the name of the method
        /// </summary>
        public string MethodName { get; set; }
        /// <summary>
        /// Gets/Sets the Method accociaded with the current Proxy Method.
        /// </summary>
        /// <remarks>
        /// This should only be used for advanced stuff as getting the meta data directly is
        /// expensive.
        /// </remarks>
        public MethodInfo Method { get; set; }
        /// <summary>
        /// Gets/Sets the Parameters used by a method.
        /// </summary>
        public List<object> MethodParameters { get; set; }
        /// <summary>
        /// Gets/Sets the exception that occured on Proxy method
        /// </summary>
        public Exception Ex { get; set; }
        /// <summary>
        /// Default constructor, that initializes the parameters collection.
        /// </summary>
        public ProxyMethod()
        {
            MethodParameters = new List<object>();
        }
    }
}
